"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var FilterDescriptor_1 = require("../models/FilterDescriptor");
var FilterGroupDescriptor_1 = require("../models/FilterGroupDescriptor");
var FilterOperator_1 = require("../enums/FilterOperator");
var FilterCondition_1 = require("../enums/FilterCondition");
var ts_commons_1 = require("ts-commons");
var class_transformer_1 = require("class-transformer");
var FilterHelper = /** @class */ (function () {
    function FilterHelper() {
    }
    FilterHelper.predicateByFilters = function (obj, filters) {
        var filterGroupDescriptor = new FilterGroupDescriptor_1.FilterGroupDescriptor();
        filterGroupDescriptor.addFilters(filters);
        return this.predicateByFilterGroupDescriptor(obj, filterGroupDescriptor);
    };
    FilterHelper.predicateByFilterGroupDescriptor = function (obj, filterGroupDescriptor) {
        var filters = filterGroupDescriptor.getFilters();
        if (ts_commons_1.ArrayUtils.isEmpty(filters)) {
            return true;
        }
        var result = null; // if filter is empty, result default value is true
        for (var _i = 0, filters_1 = filters; _i < filters_1.length; _i++) {
            var filter = filters_1[_i];
            if (result === false &&
                !ts_commons_1.ObjectUtils.isNullOrUndefined(filter) &&
                filter.getCondition() === FilterCondition_1.FilterCondition.AND) {
                continue;
            }
            if (result === true &&
                !ts_commons_1.ObjectUtils.isNullOrUndefined(filter) &&
                filter.getCondition() === FilterCondition_1.FilterCondition.OR) {
                continue;
            }
            var predictResult = void 0;
            if (filter instanceof FilterDescriptor_1.FilterDescriptor) {
                predictResult = this.predicateByFilterDescriptor(obj, filter);
            }
            else if (filter instanceof FilterGroupDescriptor_1.FilterGroupDescriptor) {
                predictResult = this.predicateByFilterGroupDescriptor(obj, filter);
            }
            else {
                predictResult = false;
            }
            if (ts_commons_1.ObjectUtils.isNullOrUndefined(result)) {
                result = predictResult;
            }
            else {
                switch (filter.getCondition()) {
                    case FilterCondition_1.FilterCondition.AND:
                        result = result && predictResult;
                        break;
                    case FilterCondition_1.FilterCondition.OR:
                        result = result || predictResult;
                        break;
                }
            }
        }
        return result;
    };
    FilterHelper.predicateByFilterDescriptor = function (obj, FilterDescriptor) {
        var operator = FilterDescriptor.operator;
        var propertyPath = FilterDescriptor.propertyPath;
        var filterValue = FilterDescriptor.value;
        var ignoreCase = FilterDescriptor.ignoreCase;
        switch (operator) {
            case FilterOperator_1.FilterOperator.EQUAL:
                return this.predicateEqual(obj, propertyPath, filterValue, ignoreCase);
            case FilterOperator_1.FilterOperator.NOT_EQUAL:
                return this.predicateNotEqual(obj, propertyPath, filterValue, ignoreCase);
            case FilterOperator_1.FilterOperator.LESS_THAN:
                return this.predicateLessThan(obj, propertyPath, filterValue);
            case FilterOperator_1.FilterOperator.LESS_THAN_OR_EQUAL:
                return this.predicateLessThanOrEqual(obj, propertyPath, filterValue);
            case FilterOperator_1.FilterOperator.GREATER_THAN_OR_EQUAL:
                return this.predicateGreaterThanOrEqual(obj, propertyPath, filterValue);
            case FilterOperator_1.FilterOperator.GREATER_THAN:
                return this.predicateGreaterThan(obj, propertyPath, filterValue);
            case FilterOperator_1.FilterOperator.START_WITH:
                return this.predicateStartWith(obj, propertyPath, filterValue, ignoreCase);
            case FilterOperator_1.FilterOperator.END_WITH:
                return this.predicateEndWith(obj, propertyPath, filterValue, ignoreCase);
            case FilterOperator_1.FilterOperator.CONTAINS:
                return this.predicateContains(obj, propertyPath, filterValue, ignoreCase);
            case FilterOperator_1.FilterOperator.IN:
                return this.predicateIn(obj, propertyPath, filterValue);
            case FilterOperator_1.FilterOperator.NOT_IN:
                return this.predicateNotIn(obj, propertyPath, filterValue);
            case FilterOperator_1.FilterOperator.BETWEEN:
                var filterValues = this.getFilterValues(FilterOperator_1.FilterOperator.BETWEEN, filterValue);
                return (this.predicateGreaterThanOrEqual(obj, propertyPath, filterValues[0]) && this.predicateLessThanOrEqual(obj, propertyPath, filterValues[1]));
            case FilterOperator_1.FilterOperator.BITWISE_ANY:
                return this.predicateBitwiseAny(obj, propertyPath, filterValue);
            case FilterOperator_1.FilterOperator.BITWISE_ZERO:
                return this.predicateBitwiseZero(obj, propertyPath, filterValue);
            case FilterOperator_1.FilterOperator.BITWISE_ALL:
                return this.predicateBitwiseAll(obj, propertyPath, filterValue);
            default:
                return false;
        }
    };
    FilterHelper.predicateEqual = function (obj, propertyPath, filterValue, ignoreCase) {
        var propValue = obj[propertyPath];
        if (typeof propValue === "string" && typeof filterValue === "string") {
            return ignoreCase
                ? ts_commons_1.StringUtils.equalsIgnoreCase(propValue, filterValue)
                : ts_commons_1.StringUtils.equals(propValue, filterValue);
        }
        else {
            var userPropValue = this.getValue(propValue);
            var useFilterValue = this.getValue(filterValue);
            return userPropValue === useFilterValue;
        }
    };
    FilterHelper.predicateNotEqual = function (obj, propertyPath, filterValue, ignoreCase) {
        return !this.predicateEqual(obj, propertyPath, filterValue, ignoreCase);
    };
    FilterHelper.predicateLessThan = function (obj, propertyPath, filterValue) {
        var propValue = obj[propertyPath];
        var userPropValue = this.getValue(propValue);
        var useFilterValue = this.getValue(filterValue);
        return userPropValue < useFilterValue;
    };
    FilterHelper.predicateLessThanOrEqual = function (obj, propertyPath, filterValue) {
        var propValue = obj[propertyPath];
        var userPropValue = this.getValue(propValue);
        var useFilterValue = this.getValue(filterValue);
        return userPropValue <= useFilterValue;
    };
    FilterHelper.predicateGreaterThanOrEqual = function (obj, propertyPath, filterValue) {
        var propValue = obj[propertyPath];
        var userPropValue = this.getValue(propValue);
        var useFilterValue = this.getValue(filterValue);
        return userPropValue >= useFilterValue;
    };
    FilterHelper.predicateGreaterThan = function (obj, propertyPath, filterValue) {
        var propValue = obj[propertyPath];
        var userPropValue = this.getValue(propValue);
        var useFilterValue = this.getValue(filterValue);
        return userPropValue > useFilterValue;
    };
    FilterHelper.predicateStartWith = function (obj, propertyPath, filterValue, ignoreCase) {
        var propValue = obj[propertyPath];
        if (ts_commons_1.ObjectUtils.isNullOrUndefined(propValue)) {
            return false;
        }
        var usePropValue = ts_commons_1.ObjectUtils.toSafeString(propValue);
        var useFilterValue = ts_commons_1.ObjectUtils.toSafeString(filterValue);
        return ignoreCase
            ? ts_commons_1.StringUtils.startWithIgnoreCase(usePropValue, useFilterValue)
            : ts_commons_1.StringUtils.startWith(usePropValue, useFilterValue);
    };
    FilterHelper.predicateEndWith = function (obj, propertyPath, filterValue, ignoreCase) {
        var propValue = obj[propertyPath];
        if (ts_commons_1.ObjectUtils.isNullOrUndefined(propValue)) {
            return false;
        }
        var usePropValue = ts_commons_1.ObjectUtils.toSafeString(propValue);
        var useFilterValue = ts_commons_1.ObjectUtils.toSafeString(filterValue);
        return ignoreCase
            ? ts_commons_1.StringUtils.endWithIgnoreCase(usePropValue, useFilterValue)
            : ts_commons_1.StringUtils.endWith(usePropValue, useFilterValue);
    };
    FilterHelper.predicateContains = function (obj, propertyPath, filterValue, ignoreCase) {
        var propValue = obj[propertyPath];
        if (ts_commons_1.ObjectUtils.isNullOrUndefined(propValue)) {
            return false;
        }
        var usePropValue = ts_commons_1.ObjectUtils.toSafeString(propValue);
        var useFilterValue = ts_commons_1.ObjectUtils.toSafeString(filterValue);
        return ignoreCase
            ? ts_commons_1.StringUtils.containsIgnoreCase(usePropValue, useFilterValue)
            : ts_commons_1.StringUtils.contains(usePropValue, useFilterValue);
    };
    FilterHelper.predicateIn = function (obj, propertyPath, filterValue) {
        var propValue = obj[propertyPath];
        var useFilterValues = this.getFilterValues(FilterOperator_1.FilterOperator.IN, filterValue);
        return ts_commons_1.ArrayUtils.contains(useFilterValues, propValue);
    };
    FilterHelper.predicateNotIn = function (obj, propertyPath, filterValue) {
        return !this.predicateIn(obj, propertyPath, filterValue);
    };
    FilterHelper.predicateBitwiseAny = function (obj, propertyPath, filterValue) {
        var propValue = obj[propertyPath];
        return (propValue & filterValue) > 0;
    };
    FilterHelper.predicateBitwiseZero = function (obj, propertyPath, filterValue) {
        var propValue = obj[propertyPath];
        return (propValue & filterValue) === 0;
    };
    FilterHelper.predicateBitwiseAll = function (obj, propertyPath, filterValue) {
        var propValue = obj[propertyPath];
        return (propValue & filterValue) === filterValue;
    };
    FilterHelper.getFilterValues = function (operator, filterValue) {
        var result = [];
        if (operator === FilterOperator_1.FilterOperator.IN ||
            operator === FilterOperator_1.FilterOperator.NOT_IN ||
            operator === FilterOperator_1.FilterOperator.BETWEEN) {
            if (ts_commons_1.ObjectUtils.isArray(filterValue)) {
                result = filterValue;
                if (operator === FilterOperator_1.FilterOperator.BETWEEN && result.length !== 2) {
                    var errMsg = 'if "BETWEEN" operator, the count of filter value must be 2';
                    throw new TypeError(errMsg);
                }
            }
            else {
                var errMsg = 'filter value of "IN" or "NOT_IN" operator must be array';
                throw new TypeError(errMsg);
            }
        }
        else {
            result.push(filterValue);
        }
        return result;
    };
    FilterHelper.getRealFilters = function (filters) {
        if (ts_commons_1.ArrayUtils.isEmpty(filters)) {
            return [];
        }
        var result = [];
        for (var _i = 0, _a = filters || []; _i < _a.length; _i++) {
            var filterBase = _a[_i];
            var filterJson = class_transformer_1.serialize(filterBase);
            switch (filterBase.type) {
                case "FilterDescriptor":
                    result.push(new FilterDescriptor_1.FilterDescriptor().fromJSON(filterJson));
                    break;
                case "FilterGroupDescriptor":
                    result.push(new FilterGroupDescriptor_1.FilterGroupDescriptor().fromJSON(filterJson));
                    break;
            }
        }
        return result;
    };
    FilterHelper.getValue = function (value) {
        return ts_commons_1.ObjectUtils.isNullOrUndefined(value) ? undefined : value;
    };
    return FilterHelper;
}());
exports.FilterHelper = FilterHelper;
//# sourceMappingURL=FilterHelper.js.map